from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split

from config.core import config
from regression_model.processing.data_manager import load_dataset, save_model
from regression_model.processing.features_preprocessing import preprocess_data


def run_training() -> None:

    # Load dataset.
    df_data = load_dataset(config.model_config.data_url)

    # Select specified columns.
    df_data = df_data[config.model_config.columns_to_select]

    # Preprocess data.
    df_data = preprocess_data(df_data)

    # Specify features and target.
    features_columns = df_data.columns.difference([config.model_config.target])
    target_column = config.model_config.target

    # Divide train and test
    X_train, X_test, y_train, y_test = train_test_split(
        df_data[features_columns],
        df_data[target_column],
        test_size=config.model_config.test_size,
        random_state=config.model_config.random_state,
    )

    # Train model.
    model = LogisticRegression()
    model.fit(X_train, y_train)

    # Persist model.
    save_model(model)


if __name__ == "__main__":
    run_training()



from pathlib import Path
from typing import List

from pydantic import BaseModel
from strictyaml import YAML, load

import regression_model

# Project Directories
PACKAGE_ROOT = Path(regression_model.__file__).resolve().parent
CONFIG_FILE_PATH = PACKAGE_ROOT / "config.yml"
TRAINED_MODEL_DIR = PACKAGE_ROOT / "trained_models"


class AppConfig(BaseModel):

    """
    Application-level config.

    """

    package_name: str


class ModelConfig(BaseModel):

    """
    Model-level config.

    """

    data_url: str
    columns_to_select: List[str]
    features: List[str]
    target: str
    random_state: int
    test_size: float


class Config(BaseModel):

    """
    Master config object.

    """

    app_config: AppConfig
    model_config: ModelConfig


def fetch_config_from_yaml() -> YAML:

    with open(CONFIG_FILE_PATH, "r") as conf_file:
        parsed_config = load(conf_file.read())
        return parsed_config


def load_and_validate_config() -> Config:

    parsed_config = fetch_config_from_yaml()

    # Specify the data attribute from the strictyaml YAML type.
    _config = Config(
        app_config=AppConfig(**parsed_config.data),
        model_config=ModelConfig(**parsed_config.data),
    )

    return _config


config = load_and_validate_config()



import pandas as pd
from typing import List, Optional, Tuple
from pydantic import BaseModel, ValidationError

from regression_model.config.core import config


def validate_inputs(input_data: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[dict]]:

    # Select specified columns.
    validated_data = input_data[config.model_config.features].copy()
    errors = None

    try:
        MultipleDataInputs(inputs=validated_data.to_dict(orient="records"))
    except ValidationError as error:
        errors = error.json()

    return validated_data, errors


class DataInputSchema(BaseModel):
    Age: int
    Sex: str
    Embarked: str


class MultipleDataInputs(BaseModel):
    inputs: List[DataInputSchema]



import joblib
import pandas as pd

from regression_model import __version__ as _version
from regression_model.config.core import TRAINED_MODEL_DIR


def load_dataset(url_path: str) -> pd.DataFrame:
    df = pd.read_csv(url_path)
    return df


def save_model(model) -> None:

    # Prepare versioned save file name
    save_file_name = f"model_{_version}.pkl"
    save_path = TRAINED_MODEL_DIR / save_file_name

    joblib.dump(model, save_path)

    # Delete old models.
    do_not_delete = ['__init__.py', save_file_name]
    for model_file in TRAINED_MODEL_DIR.iterdir():
        if model_file.name not in do_not_delete:
            model_file.unlink()


def load_model(file_name):
    return joblib.load(file_name)



import pandas as pd


def preprocess_data(df: pd.DataFrame) -> pd.DataFrame:

    categorical_arr = []

    for col, col_type in df.dtypes.iteritems():
        if col_type == "O":
            categorical_arr.append(col)
        else:
            df[col].fillna(0, inplace=True)

    df_ohe = pd.get_dummies(df, columns=categorical_arr, dummy_na=True)

    return df_ohe



import pandas as pd
import typing

from regression_model.processing.data_manager import load_model
from regression_model.processing.validation import validate_inputs
from regression_model import __version__ as _version
from regression_model.config.core import TRAINED_MODEL_DIR
from regression_model.processing.features_preprocessing import preprocess_data

model_file_name = TRAINED_MODEL_DIR / f"model_{_version}.pkl"
_model = load_model(file_name=model_file_name)


def make_prediction(input_data: typing.Union[pd.DataFrame, dict]) -> dict:

    data = pd.DataFrame(input_data)
    validated_data, errors = validate_inputs(input_data=data)
    validated_data = preprocess_data(validated_data)

    results = {"predictions": None, "version": _version, "errors": errors}

    all_features = [
        "Age",
        "Embarked_C",
        "Embarked_Q",
        "Embarked_S",
        "Embarked_nan",
        "Sex_female",
        "Sex_male",
        "Sex_nan",
    ]

    # Sync inputs.
    for feature in all_features:
        if feature not in validated_data:
            validated_data[feature] = 0
    validated_data = validated_data[all_features]

    if not errors:
        predictions = _model.predict_proba(X=validated_data)
        predictions = [x[1] for x in predictions]
        results = {"predictions": predictions, "version": _version, "errors": errors}

    # .
    return results
